package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kstarbound.world.api.BackgroundChunkView
import ru.dbotthepony.kstarbound.world.api.CHUNK_SIZE
import ru.dbotthepony.kstarbound.world.api.ForegroundChunkView
import ru.dbotthepony.kstarbound.world.api.IChunk
import ru.dbotthepony.kstarbound.world.api.IChunkCell
import ru.dbotthepony.kvector.arrays.Object2DArray

/**
 * Предоставляет доступ к чанку и его соседям
 *
 * В основном для использования в местах, где нужен не мир, а определённый чанк мира,
 * и при этом координаты проверяются относительно чанка и могут спокойно выйти за его пределы,
 * с желанием получить тайл из соседнего чанка
 */
class CellView(
	override val pos: ChunkPos,

	val left: IChunk?,
	val center: IChunk?,
	val right: IChunk?,

	val topRight: IChunk?,
	val top: IChunk?,
	val topLeft: IChunk?,

	val bottomLeft: IChunk?,
	val bottom: IChunk?,
	val bottomRight: IChunk?,
) : IChunk {
	val backgroundView = BackgroundChunkView(this)
	val foregroundView = ForegroundChunkView(this)

	override fun getCell(x: Int, y: Int): IChunkCell {
		val ix = x + CHUNK_SIZE
		val iy = y + CHUNK_SIZE

		if (ix in 0 until CHUNK_SIZE * 3 - 1 && iy in 0 until CHUNK_SIZE * 3 - 1) {
			return indices[ix, iy].invoke(this) ?: IChunkCell.Companion
		} else {
			return IChunkCell.Companion
		}
	}

	companion object {
		private val indices = Object2DArray.nulls<(CellView) -> IChunkCell?>(CHUNK_SIZE * 3, CHUNK_SIZE * 3) as Object2DArray<(CellView) -> IChunkCell?>

		private fun put(half: (CellView) -> IChunk?, x: Int, y: Int) {
			for (ix in 0 until CHUNK_SIZE) {
				for (iy in 0 until CHUNK_SIZE) {
					indices[x + ix, y + iy] = { half(it)?.getCell(ix, iy) }
				}
			}
		}

		init {
			put(CellView::bottomLeft,   0,              0)
			put(CellView::bottom,       CHUNK_SIZE,     0)
			put(CellView::bottomRight,  CHUNK_SIZE * 2, 0)

			put(CellView::left,         0,              CHUNK_SIZE)
			put(CellView::center,       CHUNK_SIZE,     CHUNK_SIZE)
			put(CellView::right,        CHUNK_SIZE * 2, CHUNK_SIZE)

			put(CellView::topLeft,      0,              CHUNK_SIZE * 2)
			put(CellView::top,          CHUNK_SIZE,     CHUNK_SIZE * 2)
			put(CellView::topRight,     CHUNK_SIZE * 2, CHUNK_SIZE * 2)
		}
	}
}
