package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kvector.api.IStruct2i
import ru.dbotthepony.kvector.vector.Vector2i

interface IChunk : ICellAccess {
	val pos: ChunkPos

	/**
	 * Возвращает псевдослучайное Long для заданной позиции
	 *
	 * Для использования в рендерах и прочих вещах, которым нужно стабильное число на основе своей позиции
	 */
	fun randomLongFor(x: Int, y: Int): Long {
		var long = (x or (pos.x shl CHUNK_SIZE_BITS)) * 738548L + (y or (pos.y shl CHUNK_SIZE_BITS)) * 2191293543L
		long = long xor 8339437585692L
		long = (long ushr 4) or (long shl 52)
		long *= 7848344324L
		long = (long ushr 12) or (long shl 44)
		return long
	}

	/**
	 * Возвращает псевдослучайное нормализированное Double для заданной позиции
	 *
	 * Для использования в рендерах и прочих вещах, которым нужно стабильное число на основе своей позиции
	 */
	fun randomDoubleFor(x: Int, y: Int): Double {
		return (randomLongFor(x, y) / 9.223372036854776E18) / 2.0 + 0.5
	}

	/**
	 * Возвращает псевдослучайное Long для заданной позиции
	 *
	 * Для использования в рендерах и прочих вещах, которым нужно стабильное число на основе своей позиции
	 */
	fun randomLongFor(pos: Vector2i) = randomLongFor(pos.x, pos.y)

	/**
	 * Возвращает псевдослучайное нормализированное Double для заданной позиции
	 *
	 * Для использования в рендерах и прочих вещах, которым нужно стабильное число на основе своей позиции
	 */
	fun randomDoubleFor(pos: Vector2i) = randomDoubleFor(pos.x, pos.y)
}
