package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kvector.api.IStruct2i

// for getting tiles directly, avoiding manual layer specification
interface ITileAccess : ICellAccess {
	// relative
	fun getTile(x: Int, y: Int): ITileState
	fun getTile(pos: IStruct2i) = getTile(pos.component1(), pos.component2())
}

interface ITileChunk : IChunk, ITileAccess

class ForegroundChunkView(private val parent: IChunk) : ITileChunk, IChunk by parent {
	override fun getTile(x: Int, y: Int): ITileState {
		return parent.getCell(x, y).foreground
	}
}

class BackgroundChunkView(private val parent: IChunk) : ITileChunk, IChunk by parent {
	override fun getTile(x: Int, y: Int): ITileState {
		return parent.getCell(x, y).background
	}
}

class ForegroundLocalizedView(override val pos: ChunkPos, private val parent: ICellAccess) : ITileChunk, ICellAccess by parent {
	override fun getTile(x: Int, y: Int): ITileState {
		return parent.getCell(pos.tileX + x, pos.tileY + y).foreground
	}
}

class BackgroundLocalizedView(override val pos: ChunkPos, private val parent: ICellAccess) : ITileChunk, ICellAccess by parent {
	override fun getTile(x: Int, y: Int): ITileState {
		return parent.getCell(pos.tileX + x, pos.tileY + y).background
	}
}

class ForegroundAccessView(private val parent: ICellAccess) : ITileAccess, ICellAccess by parent {
	override fun getTile(x: Int, y: Int): ITileState {
		return parent.getCell(x, y).foreground
	}
}

class BackgroundAccessView(private val parent: ICellAccess) : ITileAccess, ICellAccess by parent {
	override fun getTile(x: Int, y: Int): ITileState {
		return parent.getCell(x, y).background
	}
}
